package net.wanji.opt.controller.signalopt;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.dto.strategy.AddOrUpdateSceneDTO;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.vo.GreenBeltFlowStopTimeVO;
import net.wanji.opt.vo.GreenBeltKeyCrossFlowTimeVO;
import net.wanji.opt.vo.GreenBeltSpeedWidthVO;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.ws.rs.core.MediaType;
import java.util.Collections;
import java.util.List;

/**
 * @author duanruiming
 * @date 2024/11/28 16:03
 */
@Api(value = "GreenBeltController", description = "绿波干线")
@RequestMapping("/greenBelt")
@RestController
public class GreenBeltController {

    @Resource
    private GreenBeltInfoService greenBeltInfoService;

    @ApiOperation(value = "绿波协调方向路口流量停车次数", notes = "绿波协调方向路口流量停车次数", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/greenBeltCrossDetailHist")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject greenBeltCrossDetailHist(Integer greenId) {
        List<GreenBeltFlowStopTimeVO> greenBeltFlowStopTimeVOS = Collections.EMPTY_LIST;
        try {
            greenBeltFlowStopTimeVOS = greenBeltInfoService.greenBeltCrossDetailHist(greenId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("绿波协调方向路口流量停车次数异常");
        }
        return JsonViewObject.newInstance().success(greenBeltFlowStopTimeVOS);
    }

    @ApiOperation(value = "绿波带宽曲线", notes = "优化监测-绿波带宽曲线", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/greenBeltSpeedWidth")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject greenBeltSpeedWidth(Integer greenId) {
        List<GreenBeltSpeedWidthVO> greenBeltFlowStopTimeVOS = Collections.EMPTY_LIST;
        try {
            greenBeltFlowStopTimeVOS = greenBeltInfoService.greenBeltSpeedWidth(greenId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("绿波带宽曲线异常");
        }
        return JsonViewObject.newInstance().success(greenBeltFlowStopTimeVOS);
    }

    @ApiOperation(value = "绿波关键路口流量绿信比", notes = "优化监测-绿波关键路口流量绿信比", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/greenBeltKeyCrossFlowTime")
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AddOrUpdateSceneDTO.class),
    })
    public JsonViewObject greenBeltKeyCrossFlowTime(Integer greenId) {
        List<GreenBeltKeyCrossFlowTimeVO> greenBeltKeyCrossFlowTimeVOS = Collections.EMPTY_LIST;
        try {
            greenBeltKeyCrossFlowTimeVOS = greenBeltInfoService.greenBeltKeyCrossFlowTime(greenId);
        } catch (Exception e) {
            JsonViewObject.newInstance().fail("绿波带宽曲线异常");
        }
        return JsonViewObject.newInstance().success(greenBeltKeyCrossFlowTimeVOS);
    }
}
