package net.wanji.opt.service.signalopt.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.vo.GreenBeltFlowStopTimeVO;
import net.wanji.opt.vo.GreenBeltInfoVO;
import net.wanji.opt.vo.GreenBeltKeyCrossFlowTimeVO;
import net.wanji.opt.vo.GreenBeltSpeedWidthVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/12/02 13:42
 */
@Service
@Slf4j
public class GreenBeltServiceImpl implements GreenBeltInfoService {

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;


    @Override
    public List<GreenBeltKeyCrossFlowTimeVO> greenBeltKeyCrossFlowTime(Integer greenId) throws Exception {
        ObjectMapper mapper = JacksonUtils.getInstance();
        DecimalFormat df = new DecimalFormat("#.00");
        List<GreenBeltKeyCrossFlowTimeVO> tempResult = new ArrayList<>();
        List<GreenBeltKeyCrossFlowTimeVO> results = new ArrayList<>();
        LocalDate currentDate = LocalDate.now();
        LocalTime startTime = LocalTime.MIDNIGHT;
        LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);

        // 绿波优化数据
        LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
        queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
        List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
        // 随机路口编号，通过路口编号查询时间轴
        String tempCrossId = "";
        if (!CollectionUtils.isEmpty(entities)) {
            for (StrategyGreenOptHistEntity entity : entities) {
                GreenBeltKeyCrossFlowTimeVO vo = new GreenBeltKeyCrossFlowTimeVO();
                String keyCross = entity.getKeyCross();
                vo.setCrossId(keyCross);
                vo.setDirName(entity.getDir());
                Date controlTime = get5MinuteDate(entity.getControlTime());
                vo.setStartTime(controlTime);
                String crossGreenDetail = entity.getCrossGreenDetail();
                Integer cycle = entity.getCycle();
                List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = mapper.readValue(crossGreenDetail, new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {
                });
                if (!CollectionUtils.isEmpty(crossGreenDetails)) {
                    List<GreenBeltInfoVO.CrossGreenDetail> keyCrossDetail = crossGreenDetails.stream().filter(details -> StringUtils.equalsIgnoreCase(keyCross, details.getCrossId())).collect(Collectors.toList());
                    if (!CollectionUtils.isEmpty(keyCrossDetail)) {
                        GreenBeltInfoVO.CrossGreenDetail curKeyCross = keyCrossDetail.get(0);
                        tempCrossId = curKeyCross.getCrossId();
                        Double phaseStartTime = curKeyCross.getPhaseStartTime();
                        Double phaseEndTime = curKeyCross.getPhaseEndTime();
                        Double greenTimeRatio = Double.parseDouble(df.format(phaseEndTime - phaseStartTime / cycle));
                        vo.setGreenTimeRatio(greenTimeRatio);
                    }
                }
                tempResult.add(vo);
            }

            // 关键路口历史数据
            LambdaQueryWrapper<CrossDataHistPO> histQuery = new LambdaQueryWrapper<>();
            histQuery.eq(CrossDataHistPO::getCrossId, tempCrossId);
            histQuery.ge(CrossDataHistPO::getStartTime, startOfDay);
            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectList(histQuery);
            if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
                for (CrossDataHistPO crossDataHistPO : crossDataHistPOS) {
                    Date crossStartTime = crossDataHistPO.getStartTime();
                    String format = DateUtil.format(crossStartTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    Date minuteDate = get5MinuteDate(format);
                    Integer flow = crossDataHistPO.getFlow();
                    GreenBeltKeyCrossFlowTimeVO resultVO = new GreenBeltKeyCrossFlowTimeVO();
                    resultVO.setStartTime(minuteDate);
                    resultVO.setFlow(0);
                    resultVO.setGreenTimeRatio(0.0);
                    resultVO.setCrossId(tempCrossId);
                    for (GreenBeltKeyCrossFlowTimeVO result : tempResult) {
                        if (StringUtils.equalsIgnoreCase(tempCrossId, result.getCrossId())
                                && minuteDate.getTime() == result.getStartTime().getTime()) {
                            resultVO.setFlow(flow);
                            resultVO.setGreenTimeRatio(result.getGreenTimeRatio());
                        }
                    }
                    results.add(resultVO);
                }
            }
        }
        return results;
    }


    @Override
    public List<GreenBeltSpeedWidthVO> greenBeltSpeedWidth(Integer greenId) throws Exception {
        try {
            List<GreenBeltSpeedWidthVO> results = new ArrayList<>();
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            LambdaQueryWrapper<GreenwaveHistPO> greenWrapper = new LambdaQueryWrapper<>();
            greenWrapper.eq(GreenwaveHistPO::getGreenId, greenId);
            greenWrapper.ge(GreenwaveHistPO::getStartTime, startOfDay);
            // 绿波历史数据
            List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(greenWrapper);
            // 绿波优化数据
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(greenwaveHistPOS) && !CollectionUtils.isEmpty(entities)) {
                for (GreenwaveHistPO histPO : greenwaveHistPOS) {
                    Integer curGreenId = histPO.getGreenId();
                    Date curStartTime = histPO.getStartTime();
                    String curStartTimeStr = DateUtil.format(curStartTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    Date startTimeminuteDate = get5MinuteDate(curStartTimeStr);
                    String roadDirection = histPO.getRoadDirection();
                    Double speed = histPO.getSpeed();
                    for (StrategyGreenOptHistEntity entity : entities) {
                        Integer optGreenId = entity.getGreenId();
                        String controlTime = entity.getControlTime();
                        String dir = entity.getDir();
                        Date minuteDate = get5MinuteDate(controlTime);
                        Double greenWidthTime = entity.getGreenWidthTime();
                        if (Objects.equals(curGreenId, optGreenId) && StringUtils.equalsIgnoreCase(dir, roadDirection)
                                && minuteDate.getTime() == startTimeminuteDate.getTime()) {
                            GreenBeltSpeedWidthVO vo = new GreenBeltSpeedWidthVO();
                            vo.setStartTime(curStartTime);
                            vo.setSpeed(speed);
                            vo.setWidth(greenWidthTime);
                            vo.setDirName(GreenBeltDirEnum.getDesc(dir));
                            results.add(vo);
                        }

                    }
                }
            }

            return results;
        } catch (Exception e) {
            log.error("绿波带宽曲线异常：", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 将下发时间按五分钟取整
     *
     * @param controlTime
     * @return
     */
    public static Date get5MinuteDate(String controlTime) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.parse(controlTime, formatter);
        // 获取当前分钟
        int minute = dateTime.getMinute();
        // 计算需要补充的分钟数（如果 minute % 5 != 0）
        int minutesToAdd = (5 - minute % 5) % 5;
        // 添加这些分钟并返回新的时间
        LocalDateTime localDateTime = dateTime.plusMinutes(minutesToAdd).withSecond(0).withNano(0);
        ZoneId zoneId = ZoneId.systemDefault();
        Date minuteDate = Date.from(localDateTime.atZone(zoneId).toInstant());
        return minuteDate;
    }

    @Override
    public List<GreenBeltFlowStopTimeVO> greenBeltCrossDetailHist(Integer greenId) throws Exception {
        try {
            List<String> crossIds = greenwaveInfoMapper.selectCrossIdsById(greenId);
            long currentTimeMillis = System.currentTimeMillis();
            int startSecond = (int) (currentTimeMillis / 1000 - 3600);
            int endSecond = (int) (currentTimeMillis / 1000);
            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectByCrossIdsAndTimestamp(crossIds, startSecond, endSecond);
            if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
                List<GreenBeltFlowStopTimeVO> results = new ArrayList<>();
                Map<Date, List<CrossDataHistPO>> startTimeMap = crossDataHistPOS.stream().collect(Collectors.groupingBy(CrossDataHistPO::getStartTime));
                for (Map.Entry<Date, List<CrossDataHistPO>> entry : startTimeMap.entrySet()) {
                    GreenBeltFlowStopTimeVO greenBeltFlowStopTimeVO = new GreenBeltFlowStopTimeVO();
                    Date startDate = entry.getKey();
                    List<GreenBeltFlowStopTimeVO.FlowStopTimeDetail> crossList = new ArrayList<>();
                    greenBeltFlowStopTimeVO.setStartTime(startDate);
                    List<CrossDataHistPO> value = entry.getValue();
                    if (!CollectionUtils.isEmpty(value)) {
                        for (CrossDataHistPO crossDataHistPO : value) {
                            GreenBeltFlowStopTimeVO.FlowStopTimeDetail flowStopTimeDetail = new GreenBeltFlowStopTimeVO.FlowStopTimeDetail();
                            flowStopTimeDetail.setFlow(crossDataHistPO.getFlow());
                            flowStopTimeDetail.setStopTimes(crossDataHistPO.getStopTimes());
                            flowStopTimeDetail.setCrossId(crossDataHistPO.getCrossId());
                            // todo 先删除，后续通过路口距离/速度计算 距离通过卫博发送数据
                            flowStopTimeDetail.setTravelTime(0.0);
                            crossList.add(flowStopTimeDetail);
                        }
                    }
                    greenBeltFlowStopTimeVO.setDetailList(crossList);
                    results.add(greenBeltFlowStopTimeVO);
                }
                Collections.sort(results, Comparator.comparing(GreenBeltFlowStopTimeVO::getStartTime));
                return results;
            }
            return Collections.EMPTY_LIST;
        } catch (Exception e) {
            log.error("绿波协调方向路口流量停车次数查询异常：", e);
            throw new RuntimeException(e);
        }
    }
}
