package net.wanji.opt.synthesis.service.impl;

import com.alibaba.excel.util.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.dao.mapper.GreenwaveRealtimeMapper;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.synthesis.pojo.vo.GreenBeltAreaIndexVO;
import net.wanji.opt.synthesis.service.StrategyGreenBeltService;
import net.wanji.opt.vo.GreenBeltChartVO;
import net.wanji.opt.vo.GreenBeltInfoVO;
import net.wanji.opt.vo.GreenBeltStopTimesQueueLengthVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/11/25 13:53
 */
@Service
@Slf4j
public class StrategyGreenBeltServiceImpl implements StrategyGreenBeltService {

    @Resource
    StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    GreenwaveHistMapper greenwaveHistMapper;
    @Resource
    GreenwaveRealtimeMapper greenwaveRealtimeMapper;

    @Override
    public GreenBeltAreaIndexVO greenBeltAreaIndex(Integer greenId) {
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(GreenwaveRealtimePO::getGreenId, greenId);
        List<GreenwaveRealtimePO> realtimePOS = greenwaveRealtimeMapper.selectList(queryWrapper);
        GreenBeltAreaIndexVO greenBeltAreaIndexVO = new GreenBeltAreaIndexVO();
        Integer status = 1;
        if (!CollectionUtils.isEmpty(realtimePOS)) {
            List<GreenBeltAreaIndexVO.DirDetail> dirDetails = new ArrayList<>();
            for (GreenwaveRealtimePO realtimePO : realtimePOS) {
                GreenBeltAreaIndexVO.DirDetail dirDetail = new GreenBeltAreaIndexVO.DirDetail();
                String roadDirection = realtimePO.getRoadDirection();
                dirDetail.setDirName(GreenBeltDirEnum.getDesc(roadDirection));
                dirDetail.setTravelTime(realtimePO.getTrvalTime() / 60);
                dirDetail.setStopTimes(realtimePO.getStopTimes());
                status = status >= realtimePO.getStatus() ? status : realtimePO.getStatus();
                dirDetails.add(dirDetail);
            }
            greenBeltAreaIndexVO.setDirDetail(dirDetails);
            greenBeltAreaIndexVO.setStatus(status);
            greenBeltAreaIndexVO.setStrategyName("绿波带");
        }

        return greenBeltAreaIndexVO;
    }

    @Override
    public List<GreenBeltStopTimesQueueLengthVO> greenBeltStopTimeQueueLength(Integer greenId) {
        long currentTimeMillis = System.currentTimeMillis();
        LambdaQueryWrapper<GreenwaveHistPO> queryWrapper = new LambdaQueryWrapper();
        queryWrapper.eq(GreenwaveHistPO::getGreenId, greenId);
        // 查询两个小时之前的
        Integer startTime = (int) (currentTimeMillis / 1000) - 3600 * 2;
        queryWrapper.ge(GreenwaveHistPO::getStartTime, new Date(startTime * 1000L));
        List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(queryWrapper);
        List<GreenBeltStopTimesQueueLengthVO> results = new ArrayList<>();
        if (!CollectionUtils.isEmpty(greenwaveHistPOS)) {
            for (GreenwaveHistPO greenwaveHistPO : greenwaveHistPOS) {
                GreenBeltStopTimesQueueLengthVO vo = new GreenBeltStopTimesQueueLengthVO();
                vo.setGreenId(greenwaveHistPO.getGreenId());
                vo.setTimeStamp(greenwaveHistPO.getStartTime());
                vo.setHourMinute(DateUtils.format(greenwaveHistPO.getStartTime(), "HH:mm"));
                vo.setStopTimes(greenwaveHistPO.getStopTimes());
                vo.setCordQueueRatio(greenwaveHistPO.getCordQueueRatio() * 100);
                vo.setTravelTime(greenwaveHistPO.getTrvalTime());
                results.add(vo);
            }
        }
        List<GreenBeltStopTimesQueueLengthVO> distincts = results.stream().distinct().collect(Collectors.toList());
        return distincts;
    }


    @Override
    public GreenBeltChartVO selectChart(Integer greenId, Date queryDate) {
        try {
            if (Objects.isNull(queryDate)) {
                queryDate = new Date();
            }
            ObjectMapper mapper = JacksonUtils.getInstance();
            QueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("green_id", greenId);
            queryWrapper.orderByDesc("control_time");
            queryWrapper.last("LIMIT 2");
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(entities)) {
                Map<String, List<StrategyGreenOptHistEntity>> controlTimeMap = entities.stream().collect(Collectors.groupingBy(StrategyGreenOptHistEntity::getControlTime));
                // 判断下发是否是正向还是反向 相同时间两条，双向绿波
                GreenBeltInfoVO greenBeltInfoVO = new GreenBeltInfoVO();
                List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails = new ArrayList<>();
                // 相同时间，双向绿波
                if (controlTimeMap.size() == 1) {
                    setBeltInfo(mapper, entities, greenBeltInfoVO, dirGreenDetails);
                } else {
                    entities.remove(entities.size() - 1);
                    setBeltInfo(mapper, entities, greenBeltInfoVO, dirGreenDetails);
                }
                greenBeltInfoVO.setDirGreenDetails(dirGreenDetails);
                GreenBeltChartVO greenBeltChartVO = calGreenChart(greenBeltInfoVO);
                return greenBeltChartVO;
            }
        } catch (Exception e) {
            log.error("数据转化异常", e);
            throw new RuntimeException(e);
        }
        return null;
    }

    private static void setBeltInfo(ObjectMapper mapper, List<StrategyGreenOptHistEntity> entities, GreenBeltInfoVO greenBeltInfoVO, List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails) throws JsonProcessingException {
        for (StrategyGreenOptHistEntity entity : entities) {
            greenBeltInfoVO.setGreenId(entity.getGreenId());
            greenBeltInfoVO.setLength(entity.getLength());
            greenBeltInfoVO.setCycle(entity.getCycle());
            greenBeltInfoVO.setControlTime(entity.getControlTime());
            greenBeltInfoVO.setControlDuration(entity.getControlDuration());
            greenBeltInfoVO.setType(entity.getType());
            greenBeltInfoVO.setDynamic(entity.getDynamic());
            greenBeltInfoVO.setControlMethod(entity.getControlMethod());

            GreenBeltInfoVO.DirGreenDetail dirGreenDetail = new GreenBeltInfoVO.DirGreenDetail();
            dirGreenDetail.setDirType(entity.getDirType());
            dirGreenDetail.setDir(entity.getDir());
            dirGreenDetail.setMaxSpeed(entity.getMaxSpeed());
            dirGreenDetail.setMinSpeed(entity.getMinSpeed());
            dirGreenDetail.setGreenWidthTime(entity.getGreenWidthTime());
            String crossGreenDetail = entity.getCrossGreenDetail();
            List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = mapper.readValue(crossGreenDetail, new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {});
            dirGreenDetail.setCrossGreenDetailList(crossGreenDetails);
            dirGreenDetails.add(dirGreenDetail);
        }
    }

    private static GreenBeltChartVO calGreenChart(GreenBeltInfoVO greenBeltInfoVO) throws JsonProcessingException {
        ObjectMapper mapper = JacksonUtils.getInstance();
        Map<String, List<List<Double>>> crossRedTimesMap = new TreeMap<>();
        Map<String, List<List<Double>>> backCrossRedTimesMap = new TreeMap<>();
        Map<String, Double> crossGreenStartMap = new TreeMap<>();
        Map<String, Double> backCrossGreenStartMap = new TreeMap<>();
        Map<String, Double> distanceMap = new TreeMap<>();
        Map<String, Double> backDistanceMap = new TreeMap<>();
        String decideSpeed = "";
        String backDecideSpeed = "";
        String travelTime = "";
        String backtravelTime = "";
        // 处理绿波时序图数据
        List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails = greenBeltInfoVO.getDirGreenDetails();
        Double greenWidthTime = 0.0;
        Double backGreenWidthTime = 0.0;
        if (!CollectionUtils.isEmpty(dirGreenDetails)) {
            Integer cycle = greenBeltInfoVO.getCycle();
            for (GreenBeltInfoVO.DirGreenDetail dirGreenDetail : dirGreenDetails) {
                Integer dirType = dirGreenDetail.getDirType();
                if (dirType == 1) {
                    greenWidthTime = dirGreenDetail.getGreenWidthTime();
                    List<GreenBeltInfoVO.CrossGreenDetail> list = dirGreenDetail.getCrossGreenDetailList();
                    getaDouble(crossRedTimesMap, crossGreenStartMap, distanceMap, cycle, list);
                    decideSpeed = String.join("~", String.valueOf(dirGreenDetail.getMinSpeed()), String.valueOf(dirGreenDetail.getMaxSpeed()));
                    List<Double> travelTimeList = list.stream().map(GreenBeltInfoVO.CrossGreenDetail::getTravelTime).collect(Collectors.toList());
                    travelTime = mapper.writeValueAsString(travelTimeList);
                }
                if (dirType == 0) {
                    backDecideSpeed = String.join("~", String.valueOf(dirGreenDetail.getMinSpeed()), String.valueOf(dirGreenDetail.getMaxSpeed()));
                    backGreenWidthTime = dirGreenDetail.getGreenWidthTime();
                    List<GreenBeltInfoVO.CrossGreenDetail> list = dirGreenDetail.getCrossGreenDetailList();
                    Collections.reverse(list);
                    getaDouble(backCrossRedTimesMap, backCrossGreenStartMap, backDistanceMap, cycle, list);
                    List<Double> travelTimeList = list.stream().map(GreenBeltInfoVO.CrossGreenDetail::getTravelTime).collect(Collectors.toList());
                    backtravelTime = mapper.writeValueAsString(travelTimeList);
                }
            }
        }
        GreenBeltChartVO greenBeltChartVO = new GreenBeltChartVO();
        greenBeltChartVO.setGreenWidthTime(greenWidthTime);
        greenBeltChartVO.setBackGreenWidthTime(backGreenWidthTime);
        greenBeltChartVO.setCrossRedTimesMap(mapper.writeValueAsString(crossRedTimesMap));
        greenBeltChartVO.setBackCrossRedTimesMap(mapper.writeValueAsString(backCrossRedTimesMap));
        greenBeltChartVO.setGreenStartMap(mapper.writeValueAsString(crossGreenStartMap));
        greenBeltChartVO.setBackGreenStartMap(mapper.writeValueAsString(backCrossGreenStartMap));
        greenBeltChartVO.setDistanceMap(mapper.writeValueAsString(distanceMap));
        greenBeltChartVO.setDecideSpeed(decideSpeed.concat("km/h"));
        greenBeltChartVO.setBackDecideSpeed(backDecideSpeed.concat("km/h"));
        greenBeltChartVO.setTravelTime(travelTime);
        greenBeltChartVO.setBackTravelTime(backtravelTime);
        return greenBeltChartVO;
    }

    private static void getaDouble(Map<String, List<List<Double>>> crossRedTimesMap, Map<String, Double> crossGreenStartMap, Map<String, Double> distanceMap,
                                   Integer cycle, List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetailList) {
        if (!CollectionUtils.isEmpty(crossGreenDetailList)) {

            // 所有行程时间
            double totalTravel = crossGreenDetailList.stream().mapToDouble(GreenBeltInfoVO.CrossGreenDetail::getTravelTime).sum();
            GreenBeltInfoVO.CrossGreenDetail firstCrossGreenDetail = crossGreenDetailList.get(0);
            Double firstOffset = firstCrossGreenDetail.getOffset();
            Double firstPhaseStartTime = firstCrossGreenDetail.getPhaseStartTime();
            Double firstGreenStart = firstCrossGreenDetail.getGreenStartTime();
            double cycleSum = Math.ceil((totalTravel + firstOffset + firstPhaseStartTime + firstGreenStart) / cycle);
            Double crossGreenStartTime = firstOffset + firstPhaseStartTime + firstGreenStart;
            for (GreenBeltInfoVO.CrossGreenDetail crossGreenDetail : crossGreenDetailList) {
                List<List<Double>> redTimes = new ArrayList<>();
                String crossId = crossGreenDetail.getCrossId();
                Double offset = crossGreenDetail.getOffset();
                Double phaseStartTime = crossGreenDetail.getPhaseStartTime();
                Double travelTime = crossGreenDetail.getTravelTime();
                Double distance = crossGreenDetail.getDistance();
                // 相位结束时间大于周期，绿灯时时间补到周期开始位置
                Double cycleD = Double.valueOf(cycle);
                Double phaseEndTime = crossGreenDetail.getPhaseEndTime();

                for (int i = 0; i < cycleSum; i++) {
                    // 周期 + 相位差
                    Double cycleIndex = i * cycleD;
                    Double phaseReverseStartTime = phaseEndTime.intValue() - cycleD;
                    // 最后结尾要将前面相位差时间补充为周期
                    Double endCycleExt = cycleD - offset;
                    Double offsetIndex = endCycleExt;
                    // 相位结束时间 > cycle, 周期相位差后面补充周期要添加绿灯
                    if (phaseReverseStartTime > 0) {
                        phaseReverseStartTime = Math.abs(phaseReverseStartTime);
                        // cycle内红灯
                        redTimes.add(Arrays.asList(cycleIndex + phaseReverseStartTime, cycleIndex + phaseStartTime));
                        redTimes.add(Arrays.asList(cycleIndex + phaseEndTime, cycleIndex + cycleD));
                        if (endCycleExt >= phaseStartTime) {
                            // endCycleExt内红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + phaseReverseStartTime, cycleIndex + offset + phaseStartTime));
                            // offset内红灯
                            redTimes.add(Arrays.asList(phaseReverseStartTime - offsetIndex, phaseStartTime - offsetIndex));
                        } else if (endCycleExt < phaseStartTime && phaseReverseStartTime > endCycleExt) {
                            // endCycleExt全绿灯
                            // offset全绿灯
                        } else if (phaseReverseStartTime <= endCycleExt) {
                            // endCycleExt内红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + phaseReverseStartTime, cycleIndex + offset + phaseStartTime));
                            // offset内红灯
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        }
                    } else {
                        // cycle内红灯
                        redTimes.add(Arrays.asList(cycleIndex + offset, cycleIndex + offset + phaseStartTime));
                        redTimes.add(Arrays.asList(cycleIndex + offset + phaseEndTime, cycleIndex + offset + cycleD));
                        if (endCycleExt >= phaseStartTime) {
                            // endCycleExt红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + cycleD, cycleIndex + offset + cycleD + phaseStartTime));
                            // offset红灯
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        } else {
                            // endCycleExt红灯
                            redTimes.add(Arrays.asList(cycleIndex + offset + cycleD, cycleIndex + offset + cycleD + endCycleExt));
                            // offset内红灯
                            redTimes.add(Arrays.asList(0.0, phaseStartTime - offsetIndex));
                            redTimes.add(Arrays.asList(phaseEndTime - offsetIndex, offset));
                        }
                    }

                    // 去重和排序
                    List<List<Double>> uniqueSortedRedTimes = redTimes.stream()
                            .distinct()  // 去重
                            .sorted((list1, list2) -> {
                                // 按照列表第一个元素排序
                                return Double.compare(list1.get(0), list2.get(0));
                            })
                            .collect(Collectors.toList());

                    crossRedTimesMap.put(crossId, uniqueSortedRedTimes);
                }
                crossGreenStartMap.put(crossId, crossGreenStartTime);
                crossGreenStartTime += travelTime;
                distanceMap.put(crossId, distance);
            }
        }
    }
}
